/* $Id: seq.c,v 1.7 2002/09/24 20:08:39 aa5779 Exp $ */
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include "util.h"
#include "dstring.h"
#include "lists.h"
#include "automat.h"
#include "interp.h"
#define SEQ_IMP
#include "seq.h"

storage_class seq_storage_class = {seq_create, seq_destroy, 
									 seq_check, seq_get,
									 seq_put, seq_add,
									 seq_remove,
									 seq_first, seq_next,
									 seq_enditer,
									 seq_numofitems
									};

storage *seq_create(interpreter *i, char *name, char *arg)
{
	storage *ns = ALLOC(storage);
	rangedef *rdf = NULL;
	skip_spaces(arg, &arg);
	if(*arg != '[')
		recover_error("invalid sequence syntax");
	ns->data = parse_range(arg, &arg, &rdf);
	ns->class = &seq_storage_class;
	free(rdf->name);
	FREE(rdf);
	return ns;
}


void seq_destroy(interpreter *i, storage *s)
{
	free(s->data);
}

int seq_check(interpreter *i, storage *s, dstring *key)
{
	int idx = convert_to_int(DS_BODY(key));
	return  in_range(s->data, idx) >= 0;
}

dstring *seq_get(interpreter *i, storage *s, dstring *key)
{
	int idx;
	if(*DS_BODY(key) == '=')
		return ds_fromint(select_value(s->data, convert_to_int(DS_BODY(key) + 1)), 10, 0);
	idx = in_range(s->data, convert_to_int(DS_BODY(key)));
	return idx < 0 ? ds_create("") : ds_fromint(idx, 10, 0);
}

void seq_put(interpreter *i, storage *s, dstring *key, dstring *val)
{
	recover_error("can't put on a sequence");
}

void seq_add(interpreter *i, storage *s, dstring *key, dstring *val)
{
	recover_error("can't delete to a sequence");
}

void seq_remove(interpreter *i, storage *s, dstring *key)
{
	recover_error("can't remove from a sequence");
}

typedef struct range_iter
{
	range *cur_range;
	int cur_idx;
	int repeat;
} range_iter;

dstring *seq_first(interpreter *i, storage *s, void **iter)
{
	range_iter *ri;
	if(!s->data)
		return NULL;
	ri = ALLOC(range_iter);
	ri->cur_range = s->data;
	ri->cur_idx = ri->cur_range->start;
	ri->repeat = ri->cur_range->repeat;
	*iter = ri;
	return ds_fromint(ri->cur_idx, 10, 0);
}

dstring *seq_next(interpreter *i, storage *s, void **iter)
{
	range_iter *ri = *iter;
	ri->cur_idx += ri->cur_range->dither;
	if((ri->cur_range->end - ri->cur_idx) * ri->cur_range->dither < 0)
	{
		if(!--ri->repeat)
		{
			ri->cur_range++;
			ri->repeat = ri->cur_range->repeat;
		}
		if(ri->repeat < 0)
		{
			FREE(ri);
			*iter = NULL;
			return NULL;
		}
		ri->cur_idx = ri->cur_range->start;
	}
	return ds_fromint(ri->cur_idx, 10, 0);
}

void seq_enditer(interpreter *i, storage *s, void **iter)
{
	if(*iter)
		FREE((range_iter *)*iter);
	*iter = NULL;
}

int seq_numofitems(interpreter *i, storage *s)
{
	range *ri;
	int count = 0;

	for(ri = s->data; ri->repeat > 0; ri++)
	{
		if(ri->dither > 0)
			count += (ri->end - ri->start + 1) / ri->dither * ri->repeat;
		else
			count += (ri->start - ri->end + 1) / -ri->dither * ri->repeat;
	}
	return count;
}

